import requests
import logging
import time

from bs4 import BeautifulSoup
import pandas as pd


log = logging.getLogger(__name__)


def _load_page(html):
    return BeautifulSoup(html, 'html.parser')


def _find_table(page, index, whole_page):
    if whole_page:
        return page

    return page.find_all('table')[index]


def _crawl_table(html_table):
    table = []

    for tr in html_table.find_all('tr'):
        row = []
        for th in tr.find_all('th'):
            row.append(th.text.strip())

        for td in tr.find_all('td'):
            row.append(td.text.strip())

        table.append(row)

    return table


def _map_columns_names(col, official_header):
    for h in official_header:
        if col in official_header[h]:
            return h
    return '--'


def _parse_header(table, official_header):
    header = table[0]
    header = [_map_columns_names(h.lower(), official_header) for h in header]
    table[0] = header

    return table


def _run(html, official_header, index=0, whole_page=False):
    page = _load_page(html)
    html_table = _find_table(page, index, whole_page)
    table = _crawl_table(html_table)
    table = _parse_header(table, official_header)

    return pd.DataFrame(table[1:], columns=table[0])


def crawl(url, official_header, index=0, whole_page=False):
    started_at = time.time()
    log.warning('### Loading table from %s ###' % url)

    html = requests.get('http://' + url).text
    table = _run(html, official_header, index, whole_page)

    log.warning('### Finished in %.2f seconds ###' % (time.time() - started_at))
    log.warning('### Total of rows %s ###' % len(table))

    return table


def crawl_file(file_path, official_header, index=0, whole_page=False):
    started_at = time.time()
    log.warning('### Loading table from %s ###' % file_path)

    html = open(file_path)
    table = _run(html, official_header, index, whole_page)

    log.warning('### Finished in %.2f seconds ###' % (time.time() - started_at))
    log.warning('### Total of rows %s ###' % len(table))

    return table


def crawl_raw_html(html, official_header, index=0, whole_page=False):
    started_at = time.time()

    table = _run(html, official_header, index, whole_page)

    log.warning('### Finished in %.2f seconds ###' % (time.time() - started_at))
    log.warning('### Total of rows %s ###' % len(table))

    return table




from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='crawl4us',
    version='0.1.3',
    description='A Python web crawler looking wildly for tables',
    long_description=long_description,
    url='https://github.com/camilamaia/crawl4us',
    author='Camila Maia',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='sample setuptools development web scraping crawler ',

    py_modules=["crawl4us"],
    # packages=find_packages(exclude=['contrib', 'docs', 'tests']),  # Required
    install_requires=[
        'beautifulsoup4',
        'requests',
        'pandas'
    ],
    extras_require={  # Optional
        'dev': ['pylint'],
        'test': ['pytest'],
    },

    project_urls={
        'Bug Reports': 'https://github.com/camilamaia/crawl4us/issues',
        'Source': 'https://github.com/camilamaia/crawl4us',
    },
)
